/*
---------------------------------------------------------------------------
 Menu for D-PHYS installers

 - Ask for password
 - If no password is entered, autoboot after 10 seconds
 - If password entered, show a nice menu
   
 11.1.05 Beat Rubischon <rubischon@phys.ethz.ch>
---------------------------------------------------------------------------
*/

#include "menu.h"
#include "biosio.h"
#include "string.h"
#include "syslinux.h"
#include "sha1.h"

void drawbox(char top, char left, char bot, char right,char attr, char page);
char getch(char *scan);

/* create this SHA1 hash with encpw[-unix].c */
unsigned char PASSWORD[] = { 0xf5, 0x19, 0x24, 0x86, 0x02, 
			     0x7e, 0xf8, 0x9f, 0x05, 0x28, 
			     0xe8, 0x5b, 0xde, 0x07, 0x0c, 
			     0x17, 0xfe, 0xef, 0x35, 0x73 };

char toutcnt=0;

/* ----------------------------------------------- */
/* Plot numbers and final timeout after 10 seconds */
/* ----------------------------------------------- */
TIMEOUTCODE ontimeout() {
    char row,col;
    char page = getdisppage();

    getpos(&row,&col,page);

    gotoxy(23, toutcnt * 3 + 1, page);
    csprint(".", 0x74);
    switch(toutcnt) {
      case 0:
        csprint("9", 0x74);
	break;
      case 1:
        csprint("8", 0x74);
	break;
      case 2:
        csprint("7", 0x74);
	break;
      case 3:
        csprint("6", 0x74);
	break;
      case 4:
        csprint("5", 0x74);
	break;
      case 5:
        csprint("4", 0x74);
	break;
      case 6:
        csprint("3", 0x74);
	break;
      case 7:
        csprint("2", 0x74);
	break;
      case 8:
        csprint("1", 0x74);
	break;
      case 9:
        csprint("0", 0x74);
	break;
    }
    csprint(".", 0x74);

    toutcnt++;

    gotoxy(row,col,page);

    if (toutcnt > 10) {
        return CODE_ESCAPE;
    } else {
        return CODE_WAIT;
    }
}

/* ----------------------------- */
/* Ask for password with timeout */
/* ----------------------------- */
void getpass(char *str, unsigned int size)
{
    char c;
    char s;
    char *p = str;
    char page = getdisppage();
    char row,col;
    c = getch(&s);

    while ( c != '\r' && c != '\033' ) {
	switch (c) {
	case '\0':		/* Extended char prefix */
	    break;
	case '\b':
	    if ( p > str ) {
		p--;
		csprint("\b \b",0x17);
	    }
	    break;
	case '\x15':		/* Ctrl-U: kill input */
	    while ( p > str ) {
		p--;
		csprint("\b \b",0x17);
	    }
	    break;
	default:
	    if ( c >= ' ' && (unsigned int)(p-str) < size-1 ) {
	      *p++ = c;
	      getpos(&row,&col,page);
	      csprint("*", 0x17);
	      gotoxy(row,col+1,page);
	    }
	    break;
	}

	c = getch(&s);
    }
    *p = '\0';
    csprint("\r\n",0x17);
}

/* -------------------- */
/* Show password dialog */
/* -------------------- */
int askpass () {
    char page;
    char i, j;
    char bg[2];
    char pw[16];
    SHA1Context sha;
    uint8_t Message_Digest[20];

    toutcnt=0;

    page=getdisppage();

    gotoxy(1, 1, page);
    for (i=0; i < 78; i++)
      csprint(" ", 0x70);

    bg[0]=177;
    bg[1]=0;
    for (i=0; i < 21; i++) {
	gotoxy(i + 2, 1, page);
	for (j=0; j < 78; j++)
	    csprint(bg, 0x01);
    }

    gotoxy(23, 1, page);
    for (i=0; i < 78; i++)
      csprint(" ", 0x74);

    gotoxy(1, 27, page);
    csprint("D-PHYS network boot menu", 0x70);

    for (i=0; i < 6; i++) {
	gotoxy(i + 3, 2, page);
	for (j=0; j < 43; j++)
	    csprint(" ", 0x17);
	gotoxy(i + 4, 45, page);
	csprint(" ", 0x00);
    }

    gotoxy(9, 3, page);
    for (i=0; i < 42; i++)
    	    csprint(" ", 0x00);

    drawbox(3, 2, 8, 44, 0x17, page);

    gotoxy(4, 4, page);
    csprint("Please wait 10 seconds for autoboot ", 0x17);
    gotoxy(5, 4, page);
    csprint("or enter admin password for maintenance.", 0x17);

    gotoxy(7, 4, page);
    csprint("Enter admin password: ",0x17);
    getpass(pw, sizeof(pw));

    SHA1Reset(&sha);
    SHA1Input(&sha, (char *)pw, strlen(pw));
    SHA1Result(&sha, Message_Digest);
    Message_Digest[20]=0x00;

    if (strcmp(pw, "")) {
	if (strncmp(Message_Digest, PASSWORD, 20)) {
	    return 0;
	} else {
	    return 2;
	}
    } else {
	return 1;
    }
}

/* ------------ */
/* Main routine */
/* ------------ */
int menumain(char *cmdline) {
    char c=0;
    t_menuitem * curr;
    char MAIN, INSTALL, DIAG, LIFE;
    (void)cmdline;

    init_menusystem(NULL);
    set_window_size(1,1,24,78);

    // if (syslinux) reg_ontimeout(ontimeout,10,10);
    reg_ontimeout(ontimeout,10,100);

    cls();

    while (c == 0) {
        c=askpass();

	if (c == 1) {
	    cls();
	    if (syslinux) runcommand("local");
	    else csprint("local",0x07);
	    return 0;
	}
    }

    cls();

    reg_ontimeout(NULL,30000L,10);

    INSTALL = add_menu(" Installers ");
    add_item("D-PHYS Linux","D-PHYS2 based Linux installer",OPT_RUN,"dphys2",0);
    add_item("D-PHYS Windows","unattendend based Windows installer",OPT_RUN,"unattended",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);
          
    LIFE = add_menu(" Life systems ");
    add_item("Knoppix","Knoppix life system",OPT_RUN,"knoppix",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    DIAG = add_menu(" Tools ");
    add_item("Memtest/86+","Memory test",OPT_RUN,"memtest",0);
    add_item("Burn K7","Dalco's test suite for Athlons",OPT_RUN,"burnk7",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    MAIN = add_menu(" Main Menu ");
    add_item("Installers...", "Run network based installer", OPT_SUBMENU, NULL, INSTALL);
    add_item("Life systems...", "Life systems without disk access", OPT_SUBMENU, NULL, LIFE);
    add_item("Diagnostics...", "Run diagnostics tools", OPT_SUBMENU, NULL, DIAG);
    add_item("Boot system", "Boot from local disk", OPT_RUN, "local", 0);

    curr = showmenus(MAIN);
    if (curr) {
	if (curr->action == OPT_RUN) {
	    if (syslinux) runcommand(curr->data);
	    else csprint(curr->data,0x07);
	    return 1;
	}
	csprint("Error in programming!",0x07);
    }

    if (syslinux) runcommand("local");
    else csprint("local",0x07);

    return 0;
}
